# Generated by Django 2.2.4 on 2019-09-13 20:51

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("samples", "0003_auto_20190913_0303")]

    operations = [
        migrations.CreateModel(
            name="Continent",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=100)),
            ],
        ),
        migrations.AlterField(
            model_name="composer",
            name="countries",
            field=models.ManyToManyField(
                related_name="composers",
                through="samples.ComposerCountry",
                to="samples.Country",
            ),
        ),
        migrations.AlterField(
            model_name="composercountry",
            name="composer",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="composer_countries",
                to="samples.Composer",
            ),
        ),
        migrations.AlterField(
            model_name="composercountry",
            name="country",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="composer_countries",
                to="samples.Country",
            ),
        ),
        migrations.AddField(
            model_name="country",
            name="continent",
            field=models.ForeignKey(
                default=1,
                on_delete=django.db.models.deletion.CASCADE,
                to="samples.Continent",
            ),
            preserve_default=False,
        ),
    ]
