# Generated by Django 2.2.4 on 2019-09-14 02:21

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("samples", "0004_auto_20190913_2051")]

    operations = [
        migrations.AddField(
            model_name="continent",
            name="hemisphere",
            field=models.CharField(default="", max_length=20),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="country",
            name="continent",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="countries",
                to="samples.Continent",
            ),
        ),
    ]
