# Generated by Django 2.2.4 on 2019-09-13 02:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("samples", "0001_initial")]

    operations = [
        migrations.AddField(
            model_name="opera",
            name="has_piano_transcription",
            field=models.BooleanField(default=False),
        )
    ]
